//reroute a priority queue
#include <qpid/messaging/Connection.h>
#include <qpid/messaging/Session.h>
#include <qpid/messaging/Address.h>
#include <qpid/messaging/Message.h>
#include <qpid/messaging/Sender.h>
#include <qpid/messaging/Receiver.h>
using namespace qpid::messaging;
using namespace qpid::types;

Message create_request(std::string queueName, std::string exchangeName, Address responseAddress) {
	Message request;
	request.setReplyTo(responseAddress);
	request.setContentType("amqp/map");
	request.getProperties()["x-amqp-0-10.app-id"] = "qmf2";
	request.getProperties()["qmf.opcode"] = "_method_request";

	Variant::Map objectIdMap;
	objectIdMap["_object_name"] = "org.apache.qpid.broker:queue:" + queueName;

	Variant::Map argumentsMap;
	argumentsMap["request"] = 0;
	argumentsMap["useAltExchange"] = false;
	argumentsMap["exchange"] = exchangeName;

	Variant::Map contentMap;
	contentMap["_method_name"] = "reroute";
	contentMap["_object_id"] = objectIdMap;
	contentMap["_arguments"] = argumentsMap;

	printf("%s\n", Variant(contentMap).asString().c_str());
	encode(contentMap, request);
	return request;
}

void parse_response(const Message& response) {
	Variant::Map resultsMap;
	decode(response, resultsMap);
	printf("%s\n", Variant(resultsMap).asString().c_str());
}

void reroute(std::string queueName, std::string exchangeName) {
	Connection conn("localhost:10010");
	conn.open();
	Session sess = conn.createSession();

	Receiver response_queue = sess.createReceiver("#response; {create:always, delete:always}");
	Receiver response_link = sess.createReceiver(response_queue.getName() + "; {create:always, delete:always, link:{x-bindings:[{exchange:\"qmf.default.direct\", queue:" + response_queue.getName() + ", key:" + response_queue.getName() + "}]}}");

	Message request = create_request(queueName, exchangeName, response_queue.getName());

	Sender sender = response_queue.getSession().createSender("qmf.default.direct/broker");
	sender.send(request);
	sender.getSession().sync();
	sender.close();

	Message response;
	response_queue.fetch(response);
	response_queue.getSession().acknowledge();
	parse_response(response);

	response_link.close();
	response_queue.close();
	conn.close();
}

int main() {
	Connection conn("localhost:10010");
	conn.open();
	Session sess = conn.createSession();

	//Receiver receiver = sess.createReceiver("pri; {create:always, delete:always, node:{type:queue}}");
	Receiver receiver = sess.createReceiver("pri; {create:always, node:{type:queue, x-declare:{arguments:{\"x-qpid-priorities\":10}}}}");

	Sender sender = sess.createSender("pri");
	Message m;
	m.setPriority(0);
	for (int i = 0; i < 10; i++) {
		sender.send(m);
	}
	sender.getSession().sync();

	reroute("pri", "amq.fanout");

	conn.close();
}
